@@ -1,4 +1,4 @@
-use crate::{all_imports::OpSize, circuit::EinsumAxes};
+use crate::{all_imports::OpSize, circuit::EinsumAxes, util::arc_ref_clone};
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use std::{
     collections::hash_map::Entry::{self, Vacant},
@@ -293,7 +293,7 @@ pub fn einsum_nest_path(einsum: &Einsum, path: Vec<Vec<usize>>) -> Einsum {
         ))
     }
     assert!(stack.len() == 1);
-    unwrap!((**stack[0].0).clone(), Circuit::Einsum)
+    unwrap!(arc_ref_clone(&stack[0].0), Circuit::Einsum)
 }
 
 #[pyfunction]
@@ -301,7 +301,7 @@ pub fn einsum_nest_path(einsum: &Einsum, path: Vec<Vec<usize>>) -> Einsum {
 pub fn einsum_nest_optimize_py(einsum: &Einsum) -> Option<Einsum> {
     einsum_nest_optimize(
         einsum,
-        &mut OptimizationContext::new_settings_circuit(Default::default(), einsum.clone().rc()),
+        &mut OptimizationContext::new_settings_circuit(Default::default(), einsum.crc()),
     )
 }
 
@@ -1553,7 +1553,7 @@ pub fn extract_add(add: &Add, sub: &Add) -> Option<Add> {
             Entry::Vacant(_) => return None,
         }
     }
-    *counts.entry(sub.clone().rc()).or_insert(0) += 1;
+    *counts.entry(sub.crc()).or_insert(0) += 1;
 
     Some(Add::try_from_counts(&counts, add.name_cloned()).unwrap())
 }
