@@ -159,7 +159,7 @@ pub fn batch_to_concat(circuit: CircuitRc, axis: usize, num_batches: usize) -> C
 pub fn batch_einsum_py(einsum: Einsum, settings: OptimizationSettings) -> CircResult {
     batch_einsum(
         &einsum,
-        &mut OptimizationContext::new_settings_circuit(settings, einsum.clone().rc()),
+        &mut OptimizationContext::new_settings_circuit(settings, einsum.crc()),
     )
 }
 
@@ -199,7 +199,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> CircR
 
     nested_big.compiler_print();
     let mut intermediates_above_mem = vec![];
-    visit_circuit_unwrap(nested_big.clone().rc(), |c| {
+    visit_circuit_unwrap(nested_big.crc(), |c| {
         if c.info().numel_usize() > max_single_mem {
             intermediates_above_mem.push(c);
         }
@@ -234,7 +234,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> CircR
     }
 
     let mut batchheads: Vec<CircuitRc> = vec![];
-    visit_circuit_with_parents(nested_big.clone().rc(), |x, parents| {
+    visit_circuit_with_parents(nested_big.crc(), |x, parents| {
         if x.info().named_axes.values().any(|n| n == &batch_str)
             && !parents
                 .iter()
@@ -249,7 +249,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> CircR
             batch_to_concat(c.clone(), axis_of_name(c, &batch_str).unwrap(), num_batches).unwrap()
         })
         .collect();
-    let out_still_symbols = deep_map_op(nested_big.clone().rc(), |c| {
+    let out_still_symbols = deep_map_op(nested_big.crc(), |c| {
         if let Some(p) = batchheads
             .iter()
             .position(|bh| c.info().hash == bh.info().hash)
