@@ -2,7 +2,7 @@ use crate::{
     all_imports::Shape,
     circuit::{print::repr_circuit_deep_compiler, repr::RustRepr},
 };
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use rustc_hash::FxHashMap as HashMap;
 use std::io::prelude::*;
 
 use super::{
@@ -10,7 +10,6 @@ use super::{
     circuit_utils::has_leaf_symbols,
     compiler_heuristics::deep_maybe_distribute,
     compiler_strip::strip_names,
-    debugging::randn_ablate_great_grandchildren,
     deep_map_op_context,
     deep_rewrite::deep_heuristic_nest_adds,
     deep_rewrite::{compiler_simp, deep_optimize_einsums, deep_push_down_index},
