@@ -14,9 +14,7 @@ pub fn strip_names(circuit: CircuitRc, context: &mut OptimizationContext) -> Cir
         circuit.clone(),
         &|circuit, _| match &**circuit {
             Circuit::Symbol(_sym) => None,
-            _ => circuit
-                .name()
-                .map(|_| (**circuit).clone().rename(None).rc()),
+            _ => circuit.name().map(|_| circuit.clone().rename(None)),
         },
         &mut (),
         &mut context.cache.stripped_names,
