@@ -233,26 +233,6 @@ impl Einsum {
         // could be 256 bit bitmask
         self.input_axes().flatten().copied().collect()
     }
-
-    pub fn evolve(&self, args: Option<EinsumArgs>, out_axes: Option<EinsumAxes>) -> Einsum {
-        Einsum::try_new(
-            args.unwrap_or_else(|| self.args.clone()),
-            out_axes.unwrap_or_else(|| self.out_axes.clone()),
-            self.name.clone(),
-        )
-        .unwrap()
-    }
-
-    pub fn normalize_ints(&self) -> Einsum {
-        let spec = self.get_spec();
-        let spec_normalized = spec.normalize();
-        Einsum::try_from_spec(
-            &spec_normalized,
-            &self.input_circuits().collect::<Vec<CircuitRc>>(),
-            self.name_cloned(),
-        )
-        .unwrap()
-    }
 }
 
 impl CircuitNodeAutoName for Einsum {
@@ -503,6 +483,26 @@ impl Einsum {
         )
         .unwrap()
     }
+
+    pub fn evolve(&self, args: Option<EinsumArgs>, out_axes: Option<EinsumAxes>) -> Einsum {
+        Einsum::try_new(
+            args.unwrap_or_else(|| self.args.clone()),
+            out_axes.unwrap_or_else(|| self.out_axes.clone()),
+            self.name.clone(),
+        )
+        .unwrap()
+    }
+
+    pub fn normalize_ints(&self) -> Einsum {
+        let spec = self.get_spec();
+        let spec_normalized = spec.normalize();
+        Einsum::try_from_spec(
+            &spec_normalized,
+            &self.input_circuits().collect::<Vec<CircuitRc>>(),
+            self.name_cloned(),
+        )
+        .unwrap()
+    }
 }
 
 /// Add supports broadcasting
@@ -586,7 +586,7 @@ impl Add {
     }
 
     pub fn elim_few(&self) -> CircuitRc {
-        remove_add_few_input(self).unwrap_or(self.clone().rc())
+        remove_add_few_input(self).unwrap_or(self.crc())
     }
 
     pub fn to_counts(&self) -> HashMap<CircuitRc, usize> {
@@ -1480,7 +1480,7 @@ pub fn flat_concat_back(circuits: Vec<CircuitRc>) -> (Concat, Vec<CircuitRc>) {
             .map(|(c, (start, sec))| {
                 Rearrange::nrc(
                     Index::nrc(
-                        flat.clone().rc(),
+                        flat.crc(),
                         TensorIndex(vec![TensorAxisIndex::new_plain_slice(start, start + sec)]),
                         None,
                     ),
