@@ -145,7 +145,7 @@ pub fn add_pull_concat(add: &Add) -> Option<CircuitRc> {
                     .rc();
                 let concat_of_adds = split_to_concat(add_explicit, concat_axis, sections_here).rc();
                 if add.info().shape[..] != concat_of_adds.info().shape[..] {
-                    add.clone().rc().compiler_print();
+                    add.crc().compiler_print();
                     concat_of_adds.compiler_print();
                     panic!();
                 }
