@@ -244,7 +244,7 @@ impl CircuitNode for Symbol {
         _device_dtype: &TorchDeviceDtype,
     ) -> Result<Tensor, TensorEvalError> {
         Err(TensorEvalError::NotExplicitlyComputable {
-            circuit: self.clone().rc(),
+            circuit: self.crc(),
         })
     }
 
@@ -359,6 +359,10 @@ impl ScalarConstant {
     pub fn is_one(&self) -> bool {
         self.value == 1.
     }
+
+    pub fn evolve_shape(&self, shape: Shape) -> Self {
+        Self::new(self.value, shape, self.name.clone())
+    }
 }
 
 #[test]
