@@ -48,7 +48,7 @@ fn simp<'a, T: CircuitNode + Into<Circuit> + Clone>(
         if let Some(result) = f(x) {
             if **result == x.clone().c() {
                 println!("{}", stringify!(f));
-                x.clone().rc().compiler_print();
+                x.crc().compiler_print();
                 result.compiler_print();
                 panic!()
             }
@@ -356,9 +356,8 @@ pub fn deep_push_down_index(circ: CircuitRc, min_size: Option<usize>) -> Circuit
         {
             (**circ).map_or_clone(&|index: &Index| {
                 let fused = apply_fn_until_none(index, index_fuse);
-                index_elim_identity(&fused).unwrap_or_else(|| {
-                    push_down_index(&fused).unwrap_or_else(|| fused.clone().rc())
-                })
+                index_elim_identity(&fused)
+                    .unwrap_or_else(|| push_down_index(&fused).unwrap_or_else(|| fused.crc()))
             })
         } else {
             circ
