@@ -1,6 +1,6 @@
 use itertools::Itertools;
 
-use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use rustc_hash::FxHashSet as HashSet;
 use smallvec::SmallVec as Sv;
 use std::iter::zip;
 
