@@ -1,6 +1,6 @@
-use crate::hashmaps::FxHashMap as HashMap;
 use crate::lazy::GILLazyPy;
 use crate::pyo3_prelude::*;
+use rustc_hash::FxHashMap as HashMap;
 use std::iter::zip;
 
 use super::{prelude::*, CachedCircuitInfo, HashBytes};
@@ -87,7 +87,7 @@ macro_rules! gfspecs_wrap {
 
 static GENERAL_FUNCTIONS_SPECS: GILLazyPy<HashMap<String, GeneralFunctionSpec>> =
     GILLazyPy::new_py(|py| {
-        HashMap::from([
+        HashMap::from_iter([
             gfspecs_wrap!(py, "sigmoid", 0),
             gfspecs_wrap!(py, "tanh", 0),
             gfspecs_wrap!(py, "rsqrt", 0),
