@@ -59,7 +59,7 @@ pub fn generalfunction_evaluate_simple(node: &GeneralFunction) -> Option<Circuit
             Python::with_gil(|py| {
                 ScalarConstant::new(
                     evaluate_fn_dtype_device(
-                        node.clone().rc(),
+                        node.crc(),
                         TorchDeviceDtypeOp {
                             device: None,
                             dtype: Some("float64".to_owned()),
