@@ -1,42 +1,31 @@
 use std::{
+    collections::HashSet,
     fmt::{self, Debug},
     ops,
     sync::Arc,
     vec,
 };
 
-use pyo3::{exceptions::PyTypeError, pyclass::CompareOp, types::PyTuple, AsPyPointer};
+use macro_rules_attribute::apply;
+use pyo3::{exceptions::PyTypeError, types::PyTuple, AsPyPointer};
 use uuid::uuid;
 
 use super::{
-    Getter, Matcher, MatcherData, MatcherFromPy, MatcherFromPyBase, Transform, TransformFromPy,
-    Updator,
+    BoundGetter, BoundUpdater, Getter, Matcher, MatcherData, MatcherFromPyBase, MatcherRc,
+    Transform, TransformRc, Updater,
 };
 use crate::{
-    circuit::{use_rust_comp, CircuitConstructionError, CircuitRc, HashBytes},
+    circuit::{CircuitConstructionError, CircuitRc, HashBytes},
     eq_by_big_hash::EqByBigHash,
     impl_both_by_big_hash,
     pyo3_prelude::*,
-    setup_callable,
+    setup_callable, simple_default, simple_from,
     tensor_util::Slice,
-    util::{as_sorted, flip_op_result, split_first_take, EmptySingleMany as ESM, IterInto},
+    util::{
+        arc_ref_clone, arc_unwrap_or_clone, as_sorted, flip_op_result, split_first_take,
+        EmptySingleMany as ESM,
+    },
 };
-use std::collections::HashSet;
-impl From<Matcher> for IterativeMatcher {
-    fn from(x: Matcher) -> Self {
-        IterativeMatcherData::Match(x).into()
-    }
-}
-impl From<MatcherData> for IterativeMatcher {
-    fn from(x: MatcherData) -> Self {
-        Matcher::from(x).into()
-    }
-}
-impl From<MatcherFromPyBase> for IterativeMatcher {
-    fn from(m: MatcherFromPyBase) -> Self {
-        Matcher::from(m).into()
-    }
-}
 
 #[derive(Clone, FromPyObject)]
 pub enum IterativeMatcherFromPy {
@@ -46,12 +35,33 @@ pub enum IterativeMatcherFromPy {
     PyFunc(PyObject),
 }
 
-impl From<MatcherFromPyBase> for IterativeMatcherFromPy {
-    fn from(x: MatcherFromPyBase) -> Self {
-        IterativeMatcherFromPy::BaseMatch(x.into())
-    }
+#[derive(Clone, Debug)]
+pub enum IterativeMatcherData {
+    Match(Matcher),
+    Filter(FilterIterativeMatcher),
+    Chains(HashSet<ChainItem>),
+    Raw(RawIterativeMatcher),
+    PyFunc(PyObject),
 }
 
+setup_callable!(IterativeMatcher, IterativeMatcherData, IterativeMatcherFromPy, match_iterate(circuit : CircuitRc) -> IterateMatchResults);
+
+simple_from!(|x: Matcher| -> IterativeMatcher { IterativeMatcherData::Match(x).into() });
+simple_from!(|x: MatcherData| -> IterativeMatcher { Matcher::from(x).into() });
+simple_from!(|x: MatcherFromPyBase| -> IterativeMatcher { Matcher::from(x).into() });
+simple_from!(|x: MatcherFromPyBase| -> IterativeMatcherFromPy {
+    IterativeMatcherFromPy::BaseMatch(x.into())
+});
+simple_default!(IterativeMatcherFromPy {
+    MatcherFromPyBase::default().into()
+});
+simple_default!(IterativeMatcher {
+    Matcher::default().into()
+});
+simple_default!(IterativeMatcherRc {
+    IterativeMatcher::default().into()
+});
+
 impl From<IterativeMatcherFromPy> for IterativeMatcher {
     fn from(m: IterativeMatcherFromPy) -> Self {
         match m {
@@ -59,22 +69,16 @@ impl From<IterativeMatcherFromPy> for IterativeMatcher {
             IterativeMatcherFromPy::IterativeMatcher(x) => x.into(),
             // we intentionally do a matcher here as the default - if users want
             // a IterativeMatcher pyfunc, they can explicitly use IterativeMatcher
-            // factor.
+            // factory.
             IterativeMatcherFromPy::PyFunc(x) => MatcherData::PyFunc(x).into(),
         }
     }
 }
 
-impl From<IterativeMatcherFromPy> for Arc<IterativeMatcher> {
-    fn from(m: IterativeMatcherFromPy) -> Self {
-        Arc::new(m.into())
-    }
-}
-
 #[derive(Clone)]
 pub struct ChainItem {
-    first: Arc<IterativeMatcher>,
-    rest: Vec<Arc<IterativeMatcher>>,
+    first: IterativeMatcherRc,
+    rest: Vec<IterativeMatcherRc>,
     hash: HashBytes,
 }
 
@@ -97,7 +101,7 @@ impl Debug for ChainItem {
 }
 
 impl ChainItem {
-    pub fn new(first: Arc<IterativeMatcher>, rest: Vec<Arc<IterativeMatcher>>) -> Self {
+    pub fn new(first: IterativeMatcherRc, rest: Vec<IterativeMatcherRc>) -> Self {
         let mut hasher = blake3::Hasher::new();
         hasher.update(&first.hash);
         for x in &rest {
@@ -111,20 +115,11 @@ impl ChainItem {
         }
     }
 
-    pub fn last(&self) -> &Arc<IterativeMatcher> {
+    pub fn last(&self) -> &IterativeMatcherRc {
         self.rest.last().unwrap_or(&self.first)
     }
 }
 
-#[derive(Clone, Debug)]
-pub enum IterativeMatcherData {
-    Match(Matcher),
-    Filter(FilterIterativeMatcher),
-    Chains(HashSet<ChainItem>),
-    Raw(RawIterativeMatcher),
-    PyFunc(PyObject),
-}
-
 impl IterativeMatcherData {
     fn uuid(&self) -> [u8; 16] {
         match self {
@@ -174,19 +169,11 @@ impl IterativeMatcherData {
     }
 }
 
-setup_callable!(IterativeMatcher, IterativeMatcherData, IterativeMatcherFromPy, match_iterate(circuit : CircuitRc) -> IterateMatchResults);
-
-impl Default for IterativeMatcher {
-    fn default() -> Self {
-        MatcherData::Always(true).into()
-    }
-}
-
 /// Helper with some basic rules you may want to use to control your node matching iterations.
 /// TODO: include docs in py + other places as needed.
 #[derive(Clone, Debug)]
 pub struct FilterIterativeMatcher {
-    pub iterative_matcher: Arc<IterativeMatcher>,
+    pub iterative_matcher: IterativeMatcherRc,
     ///if true, stops once it has found a match
     pub term_if_matches: bool,
     /// depth at which we start matching
@@ -194,7 +181,7 @@ pub struct FilterIterativeMatcher {
     /// depth at which we stop matching
     pub end_depth: Option<u32>,
     /// terminate iterative matching if we reach a node which matches this
-    pub term_early_at: Arc<Matcher>,
+    pub term_early_at: MatcherRc,
     depth: u32,
 }
 
@@ -204,10 +191,10 @@ impl FilterIterativeMatcher {
     /// TODO: add support for a builder with defaults because this is really annoying...
     /// TODO: actually test this when builder is added!
     pub fn new_range<R: ops::RangeBounds<u32>>(
-        iterative_matcher: Arc<IterativeMatcher>,
+        iterative_matcher: IterativeMatcherRc,
         term_if_matches: bool,
         depth_range: R,
-        term_early_at: Arc<Matcher>,
+        term_early_at: MatcherRc,
     ) -> Self {
         use ops::Bound;
 
@@ -232,11 +219,11 @@ impl FilterIterativeMatcher {
     }
 
     pub fn new(
-        iterative_matcher: Arc<IterativeMatcher>,
+        iterative_matcher: IterativeMatcherRc,
         term_if_matches: bool,
         start_depth: Option<u32>,
         end_depth: Option<u32>,
-        term_early_at: Arc<Matcher>,
+        term_early_at: MatcherRc,
     ) -> Self {
         Self {
             iterative_matcher,
@@ -267,13 +254,13 @@ impl FilterIterativeMatcher {
         let before_start = self.start_depth.map(|x| self.depth < x).unwrap_or(false);
         if before_start {
             return Ok(IterateMatchResults {
-                updated: Some(Arc::new(
+                updated: Some(
                     IterativeMatcherData::Filter(FilterIterativeMatcher {
                         depth: self.depth + 1,
                         ..self.clone()
                     })
                     .into(),
-                )),
+                ),
                 finished: false,
                 found: false,
             });
@@ -304,16 +291,16 @@ impl FilterIterativeMatcher {
             self.depth
         };
         let updated = (self.end_depth.is_some() || updated.is_some()).then(|| {
-            let new = updated.unwrap_or(self.iterative_matcher.clone());
+            let new = updated
+                .map(Into::into)
+                .unwrap_or(self.iterative_matcher.clone());
 
-            Arc::new(
-                IterativeMatcherData::Filter(FilterIterativeMatcher {
-                    iterative_matcher: new,
-                    depth: new_depth,
-                    ..self.clone()
-                })
-                .into(),
-            )
+            IterativeMatcherData::Filter(FilterIterativeMatcher {
+                iterative_matcher: new,
+                depth: new_depth,
+                ..self.clone()
+            })
+            .into()
         });
 
         Ok(IterateMatchResults {
@@ -327,7 +314,8 @@ impl FilterIterativeMatcher {
 #[pyclass]
 #[derive(Clone, Debug)]
 pub struct IterateMatchResults {
-    pub updated: Option<Arc<IterativeMatcher>>,
+    #[pyo3(set, get)]
+    pub updated: Option<IterativeMatcherRc>,
     #[pyo3(set, get)]
     pub finished: bool,
     #[pyo3(set, get)]
@@ -338,32 +326,20 @@ pub struct IterateMatchResults {
 impl IterateMatchResults {
     #[new]
     #[args(updated = "None", finished = "false", found = "false")]
-    fn py_new(updated: Option<IterativeMatcher>, finished: bool, found: bool) -> Self {
+    fn new(updated: Option<IterativeMatcherRc>, finished: bool, found: bool) -> Self {
         Self {
-            updated: updated.map(Arc::new),
+            updated,
             finished,
             found,
         }
     }
 
-    #[getter]
-    #[pyo3(name = "updated")]
-    fn updated_py(&self) -> Option<IterativeMatcher> {
-        self.updated.as_ref().map(|x| (**x).clone())
-    }
-
-    #[setter]
-    #[pyo3(name = "updated")]
-    fn updated_py_set(&mut self, updated: Option<IterativeMatcher>) {
-        self.updated = updated.map(Arc::new);
-    }
-
     fn to_tup(&self) -> Py<PyTuple> {
         Python::with_gil(|py| {
             PyTuple::new(
                 py,
                 [
-                    self.updated_py().into_py(py),
+                    self.updated.clone().into_py(py),
                     self.finished.into_py(py),
                     self.found.into_py(py),
                 ],
@@ -374,21 +350,11 @@ impl IterateMatchResults {
 }
 
 impl IterativeMatcher {
-    pub fn any(matchers: Vec<Self>) -> Self {
-        IterativeMatcherData::Chains(
-            matchers
-                .into_iter()
-                .map(|x| ChainItem::new(Arc::new(x), Vec::new()))
-                .collect::<HashSet<_>>(),
-        )
-        .into()
+    pub fn or(self, other: IterativeMatcherRc) -> Self {
+        Self::any(vec![self.crc(), other])
     }
 
-    fn to_from_py(&self) -> IterativeMatcherFromPy {
-        IterativeMatcherFromPy::IterativeMatcher(self.clone())
-    }
-
-    pub(super) fn validate_matched(&self, matched: &HashSet<CircuitRc>) -> PyResult<()> {
+    pub fn validate_matched(&self, matched: &HashSet<CircuitRc>) -> PyResult<()> {
         match &self.data {
             IterativeMatcherData::Match(m) => m.validate_matched(matched),
             IterativeMatcherData::Filter(m) => m.iterative_matcher.validate_matched(matched),
@@ -401,7 +367,6 @@ impl IterativeMatcher {
             IterativeMatcherData::PyFunc(_) | IterativeMatcherData::Raw(_) => Ok(()),
         }
     }
-
     // TODO: more rust niceness funcs like the py ones!
 }
 
@@ -409,11 +374,11 @@ impl IterativeMatcher {
 impl IterativeMatcher {
     #[new]
     #[args(inps = "*")]
-    fn py_new(inps: Vec<IterativeMatcherFromPy>) -> Self {
+    fn py_new(inps: Vec<IterativeMatcherRc>) -> Self {
         match inps.into() {
             ESM::Empty => MatcherData::Always(false).into(),
-            ESM::Single(x) => x.into(),
-            ESM::Many(x) => Self::any_py(x),
+            ESM::Single(x) => arc_ref_clone(&x),
+            ESM::Many(x) => Self::any(x),
         }
     }
 
@@ -431,20 +396,20 @@ impl IterativeMatcher {
                 enum MaybeChain<'a> {
                     Chain(&'a ChainItem),
                     Slices {
-                        first: &'a Arc<IterativeMatcher>,
-                        rest: &'a [Arc<IterativeMatcher>],
+                        first: &'a IterativeMatcherRc,
+                        rest: &'a [IterativeMatcherRc],
                     },
                 }
 
                 impl<'a> MaybeChain<'a> {
-                    fn first(&'a self) -> &'a Arc<IterativeMatcher> {
+                    fn first(&'a self) -> &'a IterativeMatcherRc {
                         match self {
                             Self::Chain(x) => &x.first,
                             Self::Slices { first, .. } => first,
                         }
                     }
 
-                    fn rest(&'a self) -> &'a [Arc<IterativeMatcher>] {
+                    fn rest(&'a self) -> &'a [IterativeMatcherRc] {
                         match self {
                             Self::Chain(x) => &x.rest,
                             Self::Slices { rest, .. } => rest,
@@ -504,7 +469,7 @@ impl IterativeMatcher {
                 IterateMatchResults {
                     finished,
                     updated: (!finished && &new_items != chains)
-                        .then(|| Arc::new(IterativeMatcherData::Chains(new_items).into())),
+                        .then(|| IterativeMatcherData::Chains(new_items).into()),
                     found: any_chain_finished,
                 }
             }
@@ -521,28 +486,131 @@ impl IterativeMatcher {
         self.validate_matched(&matched)
     }
 
-    // TODO: write flatten/simplify method if we want the extra speed + niceness!
+    #[staticmethod]
+    #[args(matchers = "*")]
+    pub fn any(matchers: Vec<IterativeMatcherRc>) -> Self {
+        IterativeMatcherData::Chains(
+            matchers
+                .into_iter()
+                .map(|x| ChainItem::new(x.into(), Vec::new()))
+                .collect(),
+        )
+        .into()
+    }
 
+    #[staticmethod]
+    #[args(first, rest = "*")]
+    pub fn mk_chain(first: IterativeMatcherRc, rest: Vec<IterativeMatcherRc>) -> Self {
+        first.chain(rest)
+    }
+
+    #[staticmethod]
+    #[args(chains = "*")]
+    pub fn mk_chain_many(chains: Vec<Vec<IterativeMatcherRc>>) -> PyResult<Self> {
+        Ok(IterativeMatcherData::Chains(
+            chains
+                .into_iter()
+                .map(|mut chain| match split_first_take(&mut chain) {
+                    None => Err(PyTypeError::new_err(
+                        "Received empty tuple for a chain, we expect len >= 1",
+                    )),
+                    Some((first, rest)) => Ok(ChainItem::new(first, rest.collect())),
+                })
+                .collect::<PyResult<_>>()?,
+        )
+        .into())
+    }
+
+    #[staticmethod]
+    #[pyo3(name = "mk_func")]
+    pub(super) fn mk_func_py(f: PyObject) -> Self {
+        IterativeMatcherData::PyFunc(f).into()
+    }
+
+    #[args(others = "*")]
+    pub fn mk_or(&self, others: Vec<IterativeMatcherRc>) -> Self {
+        Self::any([self.clone().into()].into_iter().chain(others).collect())
+    }
+
+    fn __or__(&self, other: IterativeMatcherRc) -> Self {
+        self.clone().or(other)
+    }
+    fn __ror__(&self, other: IterativeMatcherRc) -> Self {
+        arc_unwrap_or_clone(other.0).or(self.crc())
+    }
+
+    // TODO: write flatten/simplify method if we want the extra speed + niceness!
+}
+
+macro_rules! dup_functions {
+    {
+        #[self_id($self_ident:ident)]
+        impl IterativeMatcher {
+            $(
+            $( #[$($meta_tt:tt)*] )*
+        //  ^~~~attributes~~~~^
+            $vis:vis fn $name:ident (
+                &self
+                $(, $arg_name:ident : $arg_ty:ty )* $(,)?
+        //      ^~~~~~~~~~~~~~~argument list!~~~~~~~~~~~^
+                )
+                $( -> $ret_ty:ty )?
+        //      ^~~~return type~~~^
+                { $($tt:tt)* }
+            )*
+
+        }
+    } => {
+        // paste is needed due to how pymethods proc macro works
+        paste::paste!{
+            #[pymethods]
+            impl IterativeMatcher {
+                $(
+                    $(#[$($meta_tt)*])*
+                    $vis fn $name(&self, $($arg_name : $arg_ty,)*) $(-> $ret_ty)* {
+                        let $self_ident = self;
+                        $($tt)*
+                    }
+                )*
+            }
+
+            #[pymethods]
+            impl Matcher {
+                $(
+                    $(#[$($meta_tt)*])*
+                    $vis fn $name(&self, $($arg_name : $arg_ty,)*) $(-> $ret_ty)* {
+                        self.to_iterative_matcher().$name($($arg_name,)*)
+                    }
+                )*
+            }
+        }
+
+    };
+}
+
+#[apply(dup_functions)]
+#[self_id(self_)]
+impl IterativeMatcher {
     #[args(
         term_if_matches = "false",
         start_depth = "None",
         end_depth = "None",
         term_early_at = "MatcherFromPyBase::Always(false).into()"
     )]
-    pub(super) fn filter(
+    pub fn filter(
         &self,
         term_if_matches: bool,
         start_depth: Option<u32>,
         end_depth: Option<u32>,
-        term_early_at: MatcherFromPy,
-    ) -> Self {
+        term_early_at: MatcherRc,
+    ) -> IterativeMatcher {
         // TODO: flatten
         IterativeMatcherData::Filter(FilterIterativeMatcher::new(
-            Arc::new(self.clone()),
+            self_.clone().into(),
             term_if_matches,
             start_depth,
             end_depth,
-            Arc::new(term_early_at.into()),
+            term_early_at,
         ))
         .into()
     }
@@ -552,13 +620,13 @@ impl IterativeMatcher {
         depth_slice = "Slice::IDENT",
         term_early_at = "MatcherFromPyBase::Always(false).into()"
     )]
-    pub(super) fn filter_sl(
+    pub fn filter_sl(
         &self,
         term_if_matches: bool,
         depth_slice: Slice,
-        term_early_at: MatcherFromPy,
+        term_early_at: MatcherRc,
     ) -> PyResult<IterativeMatcher> {
-        Ok(self.filter(
+        Ok(self_.filter(
             term_if_matches,
             flip_op_result(depth_slice.start.map(|x| x.try_into()))?,
             flip_op_result(depth_slice.stop.map(|x| x.try_into()))?,
@@ -567,10 +635,10 @@ impl IterativeMatcher {
     }
 
     #[args(rest = "*")]
-    pub(super) fn chain(&self, rest: Vec<IterativeMatcherFromPy>) -> Self {
+    pub fn chain(&self, rest: Vec<IterativeMatcherRc>) -> IterativeMatcher {
         // TODO: flatten
         IterativeMatcherData::Chains(
-            [ChainItem::new(Arc::new(self.clone()), rest.into_collect())]
+            [ChainItem::new(self_.clone().into(), rest)]
                 .into_iter()
                 .collect::<HashSet<_>>(),
         )
@@ -578,78 +646,19 @@ impl IterativeMatcher {
     }
 
     #[args(rest = "*")]
-    pub(super) fn chain_many(&self, rest: Vec<Vec<IterativeMatcherFromPy>>) -> Self {
+    pub fn chain_many(&self, rest: Vec<Vec<IterativeMatcherRc>>) -> IterativeMatcher {
         // TODO: flatten
         IterativeMatcherData::Chains(
             rest.into_iter()
-                .map(|x| ChainItem::new(Arc::new(self.clone()), x.into_collect()))
-                .collect::<HashSet<_>>(),
+                .map(|x| ChainItem::new(self_.clone().into(), x))
+                .collect(),
         )
         .into()
     }
 
-    #[staticmethod]
-    #[pyo3(name = "any")]
-    #[args(matchers = "*")]
-    pub(super) fn any_py(matchers: Vec<IterativeMatcherFromPy>) -> Self {
-        Self::any(matchers.into_collect())
-    }
-
-    #[staticmethod]
-    #[args(first, rest = "*")]
-    pub(super) fn mk_chain(
-        first: IterativeMatcherFromPy,
-        rest: Vec<IterativeMatcherFromPy>,
-    ) -> Self {
-        first.to_iterative_matcher().chain(rest)
-    }
-
-    #[staticmethod]
-    #[args(chains = "*")]
-    pub(super) fn mk_chain_many(chains: Vec<Vec<IterativeMatcherFromPy>>) -> PyResult<Self> {
-        Ok(IterativeMatcherData::Chains(
-            chains
-                .into_iter()
-                .map(|mut chain| match split_first_take(&mut chain) {
-                    None => Err(PyTypeError::new_err(
-                        "Received empty tuple for a chain, we expect len >= 1",
-                    )),
-                    Some((first, rest)) => {
-                        Ok(ChainItem::new(Arc::new(first.into()), rest.into_collect()))
-                    }
-                })
-                .collect::<PyResult<_>>()?,
-        )
-        .into())
-    }
-
-    #[staticmethod]
-    #[pyo3(name = "mk_func")]
-    #[args(items = "*")]
-    pub(super) fn mk_func_py(f: PyObject) -> Self {
-        IterativeMatcherData::PyFunc(f).into()
-    }
-
-    #[args(others = "*")]
-    pub fn mk_or(&self, others: Vec<IterativeMatcherFromPy>) -> Self {
-        Self::any(
-            [self.clone()]
-                .into_iter()
-                .chain(others.into_iter_into())
-                .collect(),
-        )
-    }
-
-    fn __or__(&self, other: IterativeMatcherFromPy) -> Self {
-        self.mk_or(vec![other])
-    }
-    fn __ror__(&self, other: IterativeMatcherFromPy) -> Self {
-        Self::any(vec![Self::py_new(vec![other]), self.clone()])
-    }
-
     #[args(fancy_validate = "Getter::default().default_fancy_validate")]
     pub fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<HashSet<CircuitRc>> {
-        Getter::default().py_get(circuit, self.to_from_py(), Some(fancy_validate))
+        Getter::default().get(circuit, self_.crc(), Some(fancy_validate))
     }
 
     #[args(fancy_validate = "Getter::default().default_fancy_validate")]
@@ -658,37 +667,41 @@ impl IterativeMatcher {
         circuit: CircuitRc,
         fancy_validate: bool,
     ) -> PyResult<Option<CircuitRc>> {
-        Getter::default().get_unique_op(circuit, self.to_from_py(), Some(fancy_validate))
+        Getter::default().get_unique_op(circuit, self_.crc(), Some(fancy_validate))
     }
 
     #[args(fancy_validate = "Getter::default().default_fancy_validate")]
     pub fn get_unique(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<CircuitRc> {
-        Getter::default().get_unique(circuit, self.to_from_py(), Some(fancy_validate))
+        Getter::default().get_unique(circuit, self_.crc(), Some(fancy_validate))
     }
 
     pub fn validate(&self, circuit: CircuitRc) -> PyResult<()> {
-        Getter::default().validate_py(circuit, self.to_from_py())
+        Getter::default().validate(circuit, self_.crc())
+    }
+
+    #[args(default_fancy_validate = "Getter::default().default_fancy_validate")]
+    pub fn getter(&self, default_fancy_validate: bool) -> BoundGetter {
+        Getter::new(default_fancy_validate).bind(self_.crc())
     }
 
-    #[pyo3(name = "update")]
     #[args(
-        transform_along_modified_path = "TransformFromPy::Transform(Transform::ident())",
-        cache_transform = "Updator::default().cache_transform",
-        cache_transform_along_modified_path = "Updator::default().cache_transform_along_modified_path",
-        cache_update = "Updator::default().cache_update",
-        fancy_validate = "Updator::default().default_fancy_validate"
+        transform_along_modified_path = "Transform::ident().into()",
+        cache_transform = "Updater::default().cache_transform",
+        cache_transform_along_modified_path = "Updater::default().cache_transform_along_modified_path",
+        cache_update = "Updater::default().cache_update",
+        fancy_validate = "Updater::default().default_fancy_validate"
     )]
-    pub(super) fn py_update(
+    pub fn update(
         &self,
         circuit: CircuitRc,
-        transform: TransformFromPy,
-        transform_along_modified_path: TransformFromPy,
+        transform: TransformRc,
+        transform_along_modified_path: TransformRc,
         cache_transform: bool,
         cache_transform_along_modified_path: bool,
         cache_update: bool,
         fancy_validate: bool,
     ) -> Result<CircuitRc, CircuitConstructionError> {
-        Updator::py_new(
+        Updater::new(
             transform,
             transform_along_modified_path,
             cache_transform,
@@ -696,6 +709,33 @@ impl IterativeMatcher {
             cache_update,
             false,
         )
-        .update(circuit, Arc::new(self.clone()), Some(fancy_validate))
+        .update(circuit, self_.crc(), Some(fancy_validate))
+    }
+
+    #[args(
+        transform_along_modified_path = "Transform::ident().into()",
+        cache_transform = "Updater::default().cache_transform",
+        cache_transform_along_modified_path = "Updater::default().cache_transform_along_modified_path",
+        cache_update = "Updater::default().cache_update",
+        default_fancy_validate = "Updater::default().default_fancy_validate"
+    )]
+    pub fn updater(
+        &self,
+        transform: TransformRc,
+        transform_along_modified_path: TransformRc,
+        cache_transform: bool,
+        cache_transform_along_modified_path: bool,
+        cache_update: bool,
+        default_fancy_validate: bool,
+    ) -> BoundUpdater {
+        Updater::new(
+            transform,
+            transform_along_modified_path,
+            cache_transform,
+            cache_transform_along_modified_path,
+            cache_update,
+            default_fancy_validate,
+        )
+        .bind(self_.crc())
     }
 }
