@@ -9,25 +9,20 @@ use std::{
 
 use pyo3::{
     exceptions::{PyRuntimeError, PyValueError},
-    pyclass::CompareOp,
     AsPyPointer,
 };
 use regex::{Captures, Regex};
 use uuid::uuid;
 
-use super::{
-    Getter, IterativeMatcher, IterativeMatcherFromPy, Transform, TransformFromPy, Updator,
-};
+use super::{IterativeMatcher, IterativeMatcherRc};
 use crate::{
     circuit::{
-        use_rust_comp, visit_circuit, CircuitConstructionError, CircuitNode, CircuitNodeUnion,
-        CircuitRc, CircuitType, HashBytes,
+        visit_circuit, CircuitConstructionError, CircuitNode, CircuitNodeUnion, CircuitRc,
+        CircuitType,
     },
-    eq_by_big_hash::EqByBigHash,
     pyo3_prelude::*,
-    setup_callable,
-    tensor_util::Slice,
-    util::{as_sorted, EmptySingleMany as ESM, IterInto},
+    setup_callable, simple_default, simple_from,
+    util::{arc_unwrap_or_clone, as_sorted, EmptySingleMany as ESM},
 };
 use std::collections::HashSet;
 
@@ -64,12 +59,6 @@ pub enum MatcherFromPyBase {
     Matcher(Matcher),
 }
 
-impl From<MatcherFromPyBase> for MatcherFromPy {
-    fn from(x: MatcherFromPyBase) -> Self {
-        MatcherFromPy::Base(x)
-    }
-}
-
 #[derive(Clone, FromPyObject)]
 pub enum MatcherFromPy {
     Base(MatcherFromPyBase),
@@ -77,12 +66,29 @@ pub enum MatcherFromPy {
     PyFunc(PyObject),
 }
 
-impl Default for MatcherFromPy {
-    fn default() -> Self {
-        MatcherFromPyBase::Always(true).into()
-    }
+#[derive(Clone, Debug)]
+pub enum MatcherData {
+    Always(bool),
+    Name(HashSet<String>),
+    Type(HashSet<CircuitType>),
+    Regex(RegexWrap),
+    EqM(HashSet<CircuitRc>),
+    Raw(RawMatcher),
+    PyFunc(PyObject),
+    Not(MatcherRc),
+    Any(Vec<MatcherRc>),
+    All(Vec<MatcherRc>),
 }
 
+setup_callable!(Matcher, MatcherData, MatcherFromPy, call (circuit : CircuitRc) -> bool);
+
+simple_from!(|x: MatcherFromPyBase| -> MatcherFromPy { MatcherFromPy::Base(x) });
+simple_from!(|x: MatcherFromPyBase| -> MatcherRc { Matcher::from(x).rc() });
+simple_default!(MatcherFromPyBase { Self::Always(true) });
+simple_default!(MatcherFromPy { MatcherFromPyBase::default().into() });
+simple_default!(Matcher { MatcherFromPy::default().into() });
+simple_default!(MatcherRc { Matcher::default().into() });
+
 impl From<MatcherFromPyBase> for Matcher {
     fn from(m: MatcherFromPyBase) -> Self {
         match m {
@@ -96,7 +102,6 @@ impl From<MatcherFromPyBase> for Matcher {
         .into()
     }
 }
-
 impl From<MatcherFromPy> for Matcher {
     fn from(m: MatcherFromPy) -> Self {
         match m {
@@ -112,20 +117,6 @@ impl MatcherFromPyBase {
     }
 }
 
-#[derive(Clone, Debug)]
-pub enum MatcherData {
-    Always(bool),
-    Name(HashSet<String>),
-    Type(HashSet<CircuitType>),
-    Regex(RegexWrap),
-    EqM(HashSet<CircuitRc>),
-    Raw(RawMatcher),
-    PyFunc(PyObject),
-    Not(Box<Matcher>),
-    Any(Vec<Matcher>),
-    All(Vec<Matcher>),
-}
-
 #[pyclass]
 #[pyo3(name = "Regex")]
 #[derive(Debug, Clone)]
@@ -245,43 +236,21 @@ impl MatcherData {
     }
 }
 
-setup_callable!(Matcher, MatcherData, MatcherFromPy, call (circuit : CircuitRc) -> bool);
-
-impl Default for IterativeMatcherFromPy {
-    fn default() -> Self {
-        MatcherFromPyBase::Always(true).into()
-    }
-}
-
-impl Default for Matcher {
-    fn default() -> Self {
-        MatcherData::Always(true).into()
-    }
-}
-
 impl Matcher {
-    pub fn and(self, other: Self) -> Self {
-        MatcherData::All(vec![self, other]).into()
+    pub fn and(self, other: MatcherRc) -> Self {
+        Self::all(vec![self.rc(), other])
     }
 
-    pub fn or(self, other: Self) -> Self {
-        MatcherData::Any(vec![self, other]).into()
+    pub fn or(self, other: MatcherRc) -> Self {
+        Self::any(vec![self.rc(), other])
     }
 
-    pub fn all(matchers: Vec<Self>) -> Self {
-        MatcherData::All(matchers).into()
-    }
-
-    pub fn any(matchers: Vec<Self>) -> Self {
-        MatcherData::Any(matchers).into()
-    }
-
-    pub fn to_iterative_matcher_from_py(&self) -> IterativeMatcherFromPy {
-        IterativeMatcherFromPy::IterativeMatcher(self.clone().into())
+    pub fn to_iterative_matcher_rc(&self) -> IterativeMatcherRc {
+        self.to_iterative_matcher().rc()
     }
     // TODO: more rust niceness funcs like the py ones!
 
-    pub(super) fn validate_matched(&self, matched: &HashSet<CircuitRc>) -> PyResult<()> {
+    pub fn validate_matched(&self, matched: &HashSet<CircuitRc>) -> PyResult<()> {
         fn run_on_set<T: Eq + hash::Hash + Debug + Clone, D: Display>(
             matcher_set: &HashSet<T>,
             found_set: &HashSet<T>,
@@ -370,11 +339,11 @@ impl Matcher {
 impl Matcher {
     #[new]
     #[args(inps = "*")]
-    fn py_new(inps: Vec<MatcherFromPy>) -> Self {
+    fn py_new(inps: Vec<MatcherRc>) -> Self {
         match inps.into() {
             ESM::Empty => MatcherData::Always(false).into(),
-            ESM::Single(x) => x.into(),
-            ESM::Many(x) => Self::any_py(x),
+            ESM::Single(x) => arc_unwrap_or_clone(x.0),
+            ESM::Many(x) => Self::any(x),
         }
     }
 
@@ -473,167 +442,50 @@ impl Matcher {
     }
 
     #[staticmethod]
-    #[pyo3(name = "all")]
     #[args(matchers = "*")]
-    pub fn all_py(matchers: Vec<MatcherFromPy>) -> Self {
-        Self::all(matchers.into_collect())
+    pub fn all(matchers: Vec<MatcherRc>) -> Self {
+        MatcherData::All(matchers).into()
     }
 
     #[staticmethod]
-    #[pyo3(name = "any")]
     #[args(matchers = "*")]
-    pub fn any_py(matchers: Vec<MatcherFromPy>) -> Self {
-        Self::any(matchers.into_collect())
+    pub fn any(matchers: Vec<MatcherRc>) -> Self {
+        MatcherData::Any(matchers).into()
     }
 
     #[pyo3(name = "mk_not")]
     pub fn not(&self) -> Self {
-        MatcherData::Not(Box::new(self.clone())).into()
+        MatcherData::Not(self.crc()).into()
     }
 
     #[args(others = "*")]
-    pub fn mk_and(&self, others: Vec<MatcherFromPy>) -> Self {
-        Self::all(
-            [self.clone()]
-                .into_iter()
-                .chain(others.into_iter_into())
-                .collect(),
-        )
+    pub fn mk_and(&self, others: Vec<MatcherRc>) -> Self {
+        Self::all([self.crc()].into_iter().chain(others).collect())
     }
 
     #[args(others = "*")]
-    pub fn mk_or(&self, others: Vec<MatcherFromPy>) -> Self {
-        Self::any(
-            [self.clone()]
-                .into_iter()
-                .chain(others.into_iter_into())
-                .collect(),
-        )
+    pub fn mk_or(&self, others: Vec<MatcherRc>) -> Self {
+        Self::any([self.crc()].into_iter().chain(others).collect())
     }
 
     fn __invert__(&self) -> Self {
         self.not()
     }
-    fn __and__(&self, other: MatcherFromPy) -> Self {
-        self.mk_and(vec![other])
+    fn __and__(&self, other: MatcherRc) -> Self {
+        self.clone().and(other)
     }
-    fn __or__(&self, other: MatcherFromPy) -> Self {
-        self.mk_or(vec![other])
+    fn __or__(&self, other: MatcherRc) -> Self {
+        self.clone().or(other)
     }
-    fn __rand__(&self, other: MatcherFromPy) -> Self {
-        Self::py_new(vec![other]).and(self.clone())
+    fn __rand__(&self, other: MatcherRc) -> Self {
+        arc_unwrap_or_clone(other.0).and(self.crc())
     }
-    fn __ror__(&self, other: MatcherFromPy) -> Self {
-        Self::py_new(vec![other]).or(self.clone())
+    fn __ror__(&self, other: MatcherRc) -> Self {
+        arc_unwrap_or_clone(other.0).or(self.crc())
     }
 
     // methods which go via iterative approach
     pub fn to_iterative_matcher(&self) -> IterativeMatcher {
         self.clone().into()
     }
-
-    #[args(
-        term_if_matches = "false",
-        start_depth = "None",
-        end_depth = "None",
-        term_early_at = "MatcherFromPyBase::Always(false).into()"
-    )]
-    fn filter(
-        &self,
-        term_if_matches: bool,
-        start_depth: Option<u32>,
-        end_depth: Option<u32>,
-        term_early_at: MatcherFromPy,
-    ) -> IterativeMatcher {
-        self.to_iterative_matcher()
-            .filter(term_if_matches, start_depth, end_depth, term_early_at)
-    }
-
-    #[args(
-        term_if_matches = "false",
-        depth_slice = "Slice::IDENT",
-        term_early_at = "MatcherFromPyBase::Always(false).into()"
-    )]
-    fn filter_sl(
-        &self,
-        term_if_matches: bool,
-        depth_slice: Slice,
-        term_early_at: MatcherFromPy,
-    ) -> PyResult<IterativeMatcher> {
-        self.to_iterative_matcher()
-            .filter_sl(term_if_matches, depth_slice, term_early_at)
-    }
-
-    #[args(rest = "*")]
-    fn chain(&self, rest: Vec<IterativeMatcherFromPy>) -> IterativeMatcher {
-        self.to_iterative_matcher().chain(rest)
-    }
-
-    #[args(rest = "*")]
-    fn chain_many(&self, rest: Vec<Vec<IterativeMatcherFromPy>>) -> IterativeMatcher {
-        self.to_iterative_matcher().chain_many(rest)
-    }
-    #[args(fancy_validate = "Getter::default().default_fancy_validate")]
-    pub fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<HashSet<CircuitRc>> {
-        Getter::default().py_get(
-            circuit,
-            self.to_iterative_matcher_from_py(),
-            Some(fancy_validate),
-        )
-    }
-
-    #[args(fancy_validate = "Getter::default().default_fancy_validate")]
-    pub fn get_unique_op(
-        &self,
-        circuit: CircuitRc,
-        fancy_validate: bool,
-    ) -> PyResult<Option<CircuitRc>> {
-        Getter::default().get_unique_op(
-            circuit,
-            self.to_iterative_matcher_from_py(),
-            Some(fancy_validate),
-        )
-    }
-
-    #[args(fancy_validate = "Getter::default().default_fancy_validate")]
-    pub fn get_unique(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<CircuitRc> {
-        Getter::default().get_unique(
-            circuit,
-            self.to_iterative_matcher_from_py(),
-            Some(fancy_validate),
-        )
-    }
-
-    pub fn validate(&self, circuit: CircuitRc) -> PyResult<()> {
-        Getter::default().validate_py(circuit, self.to_iterative_matcher_from_py())
-    }
-
-    #[pyo3(name = "update")]
-    #[args(
-        transform_along_modified_path = "TransformFromPy::Transform(Transform::ident())",
-        cache_transform = "Updator::default().cache_transform",
-        cache_transform_along_modified_path = "Updator::default().cache_transform_along_modified_path",
-        cache_update = "Updator::default().cache_update",
-        fancy_validate = "Updator::default().default_fancy_validate"
-    )]
-    fn py_update(
-        &self,
-        circuit: CircuitRc,
-        transform: TransformFromPy,
-        transform_along_modified_path: TransformFromPy,
-        cache_transform: bool,
-        cache_transform_along_modified_path: bool,
-        cache_update: bool,
-        fancy_validate: bool,
-    ) -> Result<CircuitRc, CircuitConstructionError> {
-        self.to_iterative_matcher().py_update(
-            circuit,
-            transform,
-            transform_along_modified_path,
-            cache_transform,
-            cache_transform_along_modified_path,
-            cache_update,
-            fancy_validate,
-        )
-    }
 }
