@@ -1,12 +1,9 @@
-use std::sync::Arc;
-
-use super::{
-    IterateMatchResults, IterativeMatcher, IterativeMatcherFromPy, Transform, TransformData,
-    TransformFromPy,
-};
+use super::{IterateMatchResults, IterativeMatcherRc, Transform, TransformData, TransformRc};
 use crate::{
     cached_method,
-    circuit::{CircuitConstructionError, CircuitNode, CircuitRc, HashBytes},
+    circuit::{
+        module_nodes::expand_node, CircuitConstructionError, CircuitNode, CircuitRc, HashBytes,
+    },
     pyo3_prelude::*,
 };
 use std::collections::HashSet;
@@ -16,24 +13,30 @@ use pyo3::exceptions::PyRuntimeError;
 
 #[pyclass]
 #[derive(Debug, Clone)]
-pub struct Updator {
-    pub(super) transform: Transform,
-    pub(super) transform_along_modified_path: Transform,
+pub struct Updater {
+    #[pyo3(get)]
+    pub(super) transform: TransformRc,
+    #[pyo3(get)]
+    pub(super) transform_along_modified_path: TransformRc,
+    #[pyo3(get)]
     pub(super) cache_transform: bool,
+    #[pyo3(get)]
     pub(super) cache_transform_along_modified_path: bool,
+    #[pyo3(get)]
     pub(super) cache_update: bool,
+    #[pyo3(get, set)]
     pub(super) default_fancy_validate: bool,
     pub(super) transform_cache: cached::UnboundCache<HashBytes, CircuitRc>,
     pub(super) transform_along_modified_path_cache: cached::UnboundCache<HashBytes, CircuitRc>,
-    pub(super) updated_cache: cached::UnboundCache<(HashBytes, Arc<IterativeMatcher>), CircuitRc>,
+    pub(super) updated_cache: cached::UnboundCache<(HashBytes, IterativeMatcherRc), CircuitRc>,
     pub(super) validation_getter: Getter,
 }
 
-impl Default for Updator {
+impl Default for Updater {
     fn default() -> Self {
         Self {
-            transform: Default::default(),
-            transform_along_modified_path: Default::default(),
+            transform: Transform::ident().into(),
+            transform_along_modified_path: Transform::ident().into(),
             cache_transform: true,
             cache_transform_along_modified_path: true,
             cache_update: true,
@@ -47,26 +50,26 @@ impl Default for Updator {
 }
 
 #[pymethods]
-impl Updator {
+impl Updater {
     #[new]
     #[args(
-        transform_along_modified_path = "TransformFromPy::Transform(Transform::ident())",
-        cache_transform = "Updator::default().cache_transform",
-        cache_transform_along_modified_path = "Updator::default().cache_transform_along_modified_path",
-        cache_update = "Updator::default().cache_update",
-        default_fancy_validate = "Updator::default().default_fancy_validate"
+        transform_along_modified_path = "Transform::ident().into()",
+        cache_transform = "Updater::default().cache_transform",
+        cache_transform_along_modified_path = "Updater::default().cache_transform_along_modified_path",
+        cache_update = "Updater::default().cache_update",
+        default_fancy_validate = "Updater::default().default_fancy_validate"
     )]
-    pub(super) fn py_new(
-        transform: TransformFromPy,
-        transform_along_modified_path: TransformFromPy,
+    pub fn new(
+        transform: TransformRc,
+        transform_along_modified_path: TransformRc,
         cache_transform: bool,
         cache_transform_along_modified_path: bool,
         cache_update: bool,
         default_fancy_validate: bool,
     ) -> Self {
         Self {
-            transform: transform.into(),
-            transform_along_modified_path: transform_along_modified_path.into(),
+            transform,
+            transform_along_modified_path,
             cache_transform,
             cache_transform_along_modified_path,
             cache_update,
@@ -79,28 +82,16 @@ impl Updator {
         &mut self,
         _py: Python<'_>,
         circuit: CircuitRc,
-        matcher: IterativeMatcherFromPy,
+        matcher: IterativeMatcherRc,
         fancy_validate: Option<bool>,
     ) -> Result<CircuitRc, CircuitConstructionError> {
-        self.py_update(circuit, matcher, fancy_validate)
+        self.update(circuit, matcher, fancy_validate)
     }
 
-    #[pyo3(name = "update")]
-    pub(super) fn py_update(
+    pub fn update(
         &mut self,
         circuit: CircuitRc,
-        matcher: IterativeMatcherFromPy,
-        fancy_validate: Option<bool>,
-    ) -> Result<CircuitRc, CircuitConstructionError> {
-        self.update(circuit, Arc::new(matcher.into()), fancy_validate)
-    }
-}
-
-impl Updator {
-    pub(super) fn update(
-        &mut self,
-        circuit: CircuitRc,
-        matcher: Arc<IterativeMatcher>,
+        matcher: IterativeMatcherRc,
         fancy_validate: Option<bool>,
     ) -> Result<CircuitRc, CircuitConstructionError> {
         if fancy_validate.unwrap_or(self.default_fancy_validate) {
@@ -110,6 +101,15 @@ impl Updator {
         self.update_impl(circuit, matcher)
     }
 
+    pub fn bind(&self, matcher: IterativeMatcherRc) -> BoundUpdater {
+        BoundUpdater {
+            updater: self.clone(),
+            matcher,
+        }
+    }
+}
+
+impl Updater {
     #[apply(cached_method)]
     #[self_id(self_)]
     #[key((circuit.info().hash, matcher.clone()))]
@@ -118,7 +118,7 @@ impl Updator {
     fn update_impl(
         &mut self,
         circuit: CircuitRc,
-        matcher: Arc<IterativeMatcher>,
+        matcher: IterativeMatcherRc,
     ) -> Result<CircuitRc, CircuitConstructionError> {
         let IterateMatchResults {
             updated,
@@ -167,20 +167,54 @@ impl Updator {
     }
 }
 
+#[pyclass]
+#[derive(Debug, Clone)]
+pub struct BoundUpdater {
+    #[pyo3(get, set)]
+    pub updater: Updater,
+    #[pyo3(get, set)]
+    pub matcher: IterativeMatcherRc,
+}
+
+#[pymethods]
+impl BoundUpdater {
+    #[new]
+    pub fn new(updater: Updater, matcher: IterativeMatcherRc) -> Self {
+        Self { updater, matcher }
+    }
+
+    fn __call__(
+        &mut self,
+        _py: Python<'_>,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+    ) -> Result<CircuitRc, CircuitConstructionError> {
+        self.update(circuit, fancy_validate)
+    }
+
+    pub fn update(
+        &mut self,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+    ) -> Result<CircuitRc, CircuitConstructionError> {
+        self.updater
+            .update(circuit, self.matcher.clone(), fancy_validate)
+    }
+}
+
 #[pyclass]
 #[derive(Debug, Clone)]
 pub struct Getter {
+    #[pyo3(get, set)]
     pub(super) default_fancy_validate: bool,
-    pub(super) cache: cached::UnboundCache<(HashBytes, Arc<IterativeMatcher>), HashSet<CircuitRc>>,
+    pub(super) cache: cached::UnboundCache<(HashBytes, IterativeMatcherRc), HashSet<CircuitRc>>,
 }
 
 impl Default for Getter {
     fn default() -> Self {
         Self {
             default_fancy_validate: false,
-            cache:
-                cached::UnboundCache::<(HashBytes, Arc<IterativeMatcher>), HashSet<CircuitRc>>::new(
-                ),
+            cache: cached::UnboundCache::new(),
         }
     }
 }
@@ -189,7 +223,7 @@ impl Default for Getter {
 impl Getter {
     #[new]
     #[args(default_fancy_validate = "Getter::default().default_fancy_validate")]
-    fn py_new(default_fancy_validate: bool) -> Self {
+    pub fn new(default_fancy_validate: bool) -> Self {
         Self {
             default_fancy_validate,
             ..Default::default()
@@ -200,65 +234,16 @@ impl Getter {
         &mut self,
         _py: Python<'_>,
         circuit: CircuitRc,
-        matcher: IterativeMatcherFromPy,
+        matcher: IterativeMatcherRc,
         fancy_validate: Option<bool>,
     ) -> PyResult<HashSet<CircuitRc>> {
-        self.py_get(circuit, matcher, fancy_validate)
+        self.get(circuit, matcher, fancy_validate)
     }
 
-    #[pyo3(name = "get")]
-    pub(super) fn py_get(
+    pub fn get(
         &mut self,
         circuit: CircuitRc,
-        matcher: IterativeMatcherFromPy,
-        fancy_validate: Option<bool>,
-    ) -> PyResult<HashSet<CircuitRc>> {
-        self.get(circuit, Arc::new(matcher.into()), fancy_validate)
-    }
-
-    pub(super) fn get_unique_op(
-        &mut self,
-        circuit: CircuitRc,
-        matcher: IterativeMatcherFromPy,
-        fancy_validate: Option<bool>,
-    ) -> PyResult<Option<CircuitRc>> {
-        let out = self.py_get(circuit, matcher, fancy_validate)?;
-        if out.len() > 1 {
-            return Err(PyRuntimeError::new_err(format!(
-                "found {} matches which is > 1",
-                out.len()
-            )));
-        }
-        Ok(out.into_iter().next())
-    }
-
-    pub(super) fn get_unique(
-        &mut self,
-        circuit: CircuitRc,
-        matcher: IterativeMatcherFromPy,
-        fancy_validate: Option<bool>,
-    ) -> PyResult<CircuitRc> {
-        self.get_unique_op(circuit, matcher, fancy_validate)?
-            .ok_or_else(|| PyRuntimeError::new_err(format!("found no matches!")))
-    }
-
-    #[pyo3(name = "validate")]
-    pub(super) fn validate_py(
-        &mut self,
-        circuit: CircuitRc,
-        matcher: IterativeMatcherFromPy,
-    ) -> PyResult<()> {
-        self.validate(circuit, Arc::new(matcher.into()))
-    }
-
-    // TODO: add support for paths as needed!
-}
-
-impl Getter {
-    pub(super) fn get(
-        &mut self,
-        circuit: CircuitRc,
-        matcher: Arc<IterativeMatcher>,
+        matcher: IterativeMatcherRc,
         fancy_validate: Option<bool>,
     ) -> PyResult<HashSet<CircuitRc>> {
         let out = self.get_impl(circuit, matcher.clone())?;
@@ -268,15 +253,48 @@ impl Getter {
         Ok(out)
     }
 
-    pub(super) fn validate(
+    pub fn get_unique_op(
         &mut self,
         circuit: CircuitRc,
-        matcher: Arc<IterativeMatcher>,
-    ) -> PyResult<()> {
+        matcher: IterativeMatcherRc,
+        fancy_validate: Option<bool>,
+    ) -> PyResult<Option<CircuitRc>> {
+        let out = self.get(circuit, matcher, fancy_validate)?;
+        if out.len() > 1 {
+            return Err(PyRuntimeError::new_err(format!(
+                "found {} matches which is > 1",
+                out.len()
+            )));
+        }
+        Ok(out.into_iter().next())
+    }
+
+    pub fn get_unique(
+        &mut self,
+        circuit: CircuitRc,
+        matcher: IterativeMatcherRc,
+        fancy_validate: Option<bool>,
+    ) -> PyResult<CircuitRc> {
+        self.get_unique_op(circuit, matcher, fancy_validate)?
+            .ok_or_else(|| PyRuntimeError::new_err(format!("found no matches!")))
+    }
+
+    pub fn validate(&mut self, circuit: CircuitRc, matcher: IterativeMatcherRc) -> PyResult<()> {
         self.get(circuit, matcher, Some(true))?;
         Ok(())
     }
 
+    pub fn bind(&self, matcher: IterativeMatcherRc) -> BoundGetter {
+        BoundGetter {
+            getter: self.clone(),
+            matcher,
+        }
+    }
+
+    // TODO: add support for paths as needed!
+}
+
+impl Getter {
     #[apply(cached_method)]
     #[self_id(self_)]
     #[key((circuit.info().hash, matcher.clone()))]
@@ -285,7 +303,7 @@ impl Getter {
     fn get_impl(
         &mut self,
         circuit: CircuitRc,
-        matcher: Arc<IterativeMatcher>,
+        matcher: IterativeMatcherRc,
     ) -> PyResult<HashSet<CircuitRc>> {
         let IterateMatchResults {
             updated,
@@ -306,3 +324,205 @@ impl Getter {
         Ok(out)
     }
 }
+
+#[pyclass]
+#[derive(Debug, Clone)]
+pub struct BoundGetter {
+    #[pyo3(get, set)]
+    pub getter: Getter,
+    #[pyo3(get, set)]
+    pub matcher: IterativeMatcherRc,
+}
+
+#[pymethods]
+impl BoundGetter {
+    #[new]
+    pub fn new(getter: Getter, matcher: IterativeMatcherRc) -> Self {
+        Self {
+            getter,
+            matcher: matcher.into(),
+        }
+    }
+
+    fn __call__(
+        &mut self,
+        _py: Python<'_>,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+    ) -> PyResult<HashSet<CircuitRc>> {
+        self.get(circuit, fancy_validate)
+    }
+
+    pub fn get(
+        &mut self,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+    ) -> PyResult<HashSet<CircuitRc>> {
+        self.getter
+            .get(circuit, self.matcher.clone(), fancy_validate)
+    }
+
+    pub fn get_unique_op(
+        &mut self,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+    ) -> PyResult<Option<CircuitRc>> {
+        self.getter
+            .get_unique_op(circuit, self.matcher.clone(), fancy_validate)
+    }
+
+    pub fn get_unique(
+        &mut self,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+    ) -> PyResult<CircuitRc> {
+        self.getter
+            .get_unique(circuit, self.matcher.clone(), fancy_validate)
+    }
+
+    pub fn validate(&mut self, circuit: CircuitRc) -> PyResult<()> {
+        self.getter.validate(circuit, self.matcher.clone())
+    }
+}
+
+#[pyclass]
+#[derive(Debug, Clone)]
+pub struct Batcher {
+    /// Note: we don't currently cache these transforms individually. We could
+    /// do this.
+    #[pyo3(get)]
+    pub(super) replacements: Vec<TransformRc>,
+    /// Technically, having all of these matchers stored here isn't important
+    /// for key functionality (like unused for caching).
+    /// This is just nice for calling from python.
+    ///
+    /// invariant: replacements.len() == matchers.len()
+    #[pyo3(get)]
+    pub(super) matchers: Vec<IterativeMatcherRc>,
+    #[pyo3(set, get)]
+    pub ban_multiple_matches_on_node: bool,
+    #[pyo3(set, get)]
+    pub default_fancy_validate: bool,
+    #[pyo3(get)]
+    pub(super) suffix: Option<String>,
+    pub(super) batch_cache: cached::UnboundCache<(HashBytes, Vec<IterativeMatcherRc>), CircuitRc>,
+    pub(super) validation_getter: Getter,
+}
+
+impl Default for Batcher {
+    fn default() -> Self {
+        Self {
+            replacements: Vec::new(),
+            matchers: Vec::new(),
+            ban_multiple_matches_on_node: false,
+            default_fancy_validate: false,
+            suffix: None,
+            batch_cache: cached::UnboundCache::new(),
+            validation_getter: Default::default(),
+        }
+    }
+}
+
+#[pymethods]
+impl Batcher {
+    #[new]
+    #[args(
+        batchers = "*",
+        ban_multiple_matches_on_node = "Batcher::default().ban_multiple_matches_on_node",
+        default_fancy_validate = "Batcher::default().default_fancy_validate",
+        suffix = "None"
+    )]
+    pub fn new(
+        batchers: Vec<(IterativeMatcherRc, TransformRc)>,
+        ban_multiple_matches_on_node: bool,
+        default_fancy_validate: bool,
+        suffix: Option<String>,
+    ) -> Self {
+        let (matchers, replacements) = batchers
+            .into_iter()
+            .map(|(a, b)| (a.into(), b.into()))
+            .unzip();
+        Self {
+            replacements,
+            matchers,
+            ban_multiple_matches_on_node,
+            default_fancy_validate,
+            suffix,
+            ..Default::default()
+        }
+    }
+
+    fn __call__(
+        &mut self,
+        _py: Python<'_>,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+    ) -> PyResult<CircuitRc> {
+        self.batch(circuit, fancy_validate)
+    }
+
+    pub fn batch(
+        &mut self,
+        circuit: CircuitRc,
+        fancy_validate: Option<bool>,
+    ) -> PyResult<CircuitRc> {
+        if fancy_validate.unwrap_or(self.default_fancy_validate) {
+            for m in &self.matchers {
+                self.validation_getter
+                    .validate(circuit.clone(), m.clone())?;
+            }
+        }
+        self.batch_impl(circuit, self.matchers.clone())
+    }
+}
+
+impl Batcher {
+    #[apply(cached_method)]
+    #[self_id(self_)]
+    #[key((circuit.info().hash, matchers.clone()))]
+    #[use_try]
+    #[cache_expr(batch_cache)]
+    fn batch_impl(
+        &mut self,
+        circuit: CircuitRc,
+        matchers: Vec<IterativeMatcherRc>,
+    ) -> PyResult<CircuitRc> {
+        let results = matchers
+            .iter()
+            .map(|m| m.match_iterate(circuit.clone()))
+            .collect::<PyResult<Vec<_>>>()?;
+
+        let filtered = results.iter().enumerate().filter(|(_, res)| res.found);
+
+        if let Some((idx, _)) = filtered.clone().next() {
+            if self_.ban_multiple_matches_on_node {
+                let n_matches = filtered.count();
+                if n_matches != 1 {
+                    return Err(PyRuntimeError::new_err(format!(
+                        "multiple matches! got {} != 1",
+                        n_matches
+                    )));
+                }
+            }
+
+            return self_.replacements[idx].run(circuit);
+        }
+
+        if results.iter().all(|x| x.finished) {
+            return Ok(circuit);
+        }
+
+        let new_matchers: Vec<_> = results
+            .into_iter()
+            .zip(matchers)
+            .map(|(res, matcher)| res.updated.unwrap_or(matcher))
+            .collect();
+
+        let new_children = circuit
+            .children()
+            .map(|c| self_.batch_impl(c, new_matchers.clone()))
+            .collect::<PyResult<_>>()?;
+
+        Ok(expand_node(circuit, &new_children)?.add_suffix(self_.suffix.as_ref().map(|x| &**x)))
+    }
+}
