@@ -1,8 +1,9 @@
 #![allow(clippy::borrow_deref_ref)]
 use crate::all_imports::{RInnerInts, Sv, TensorInvariantError};
 use crate::circuit::circuit_utils::toposort_circuit;
+use crate::py_types::use_rust_comp;
 use crate::pyo3_prelude::*;
-use crate::util::AxisInt;
+use crate::util::{arc_ref_clone, arc_unwrap_or_clone, AxisInt};
 use crate::{
     cached_lambda,
     eq_by_big_hash::EqByBigHash,
@@ -254,6 +255,12 @@ pub trait CircuitNode: CircuitNodeInit {
     // FUNCTIONS BELOW HERE *shouldn't* be overridden by implementors!
     // (if you do implement, this won't be picked up on by union types!)
 
+    fn crc(&self) -> CircuitRc
+    where
+        Self: Clone,
+    {
+        self.clone().rc()
+    }
     fn name_cloned(&self) -> Option<String> {
         self.name().map(|x| x.to_owned())
     }
@@ -394,7 +401,7 @@ pub trait CircuitNode: CircuitNodeInit {
     {
         let axes = self.check_canon_axes(axes)?;
         Ok(Einsum::new(
-            vec![(self.clone().rc(), (0u8..self.info().rank() as u8).collect())],
+            vec![(self.crc(), (0u8..self.info().rank() as u8).collect())],
             (0u8..self.info().rank() as u8)
                 .filter(|i| !axes.contains(&i))
                 .collect(),
@@ -436,7 +443,7 @@ pub trait CircuitNode: CircuitNodeInit {
 
         Ok(GeneralFunction::new_by_name(
             vec![Rearrange::nrc(
-                self.clone().rc(),
+                self.crc(),
                 RearrangeSpec::combine_axes_at_end(self.info().rank() as u8, axes),
                 None,
             )],
@@ -462,7 +469,7 @@ pub trait CircuitNode: CircuitNodeInit {
     where
         Self: Clone,
     {
-        Add::try_new(vec![self.clone().rc(), other], name)
+        Add::try_new(vec![self.crc(), other], name)
     }
     fn sub(&self, other: CircuitRc, name: Option<String>) -> Result<Add, CircuitConstructionError>
     where
@@ -478,7 +485,7 @@ pub trait CircuitNode: CircuitNodeInit {
     where
         Self: Clone,
     {
-        Einsum::elementwise_broadcasted(vec![self.clone().rc(), other], name)
+        Einsum::elementwise_broadcasted(vec![self.crc(), other], name)
     }
     fn mul_scalar(
         &self,
@@ -499,7 +506,19 @@ pub trait CircuitNode: CircuitNodeInit {
     where
         Self: Clone,
     {
-        Index::try_new(self.clone().rc(), index, name)
+        Index::try_new(self.crc(), index, name)
+    }
+    fn add_suffix(self, suffix: Option<&str>) -> Self
+    where
+        Self: Sized,
+    {
+        match suffix {
+            Some(suffix) => {
+                let name = self.name().map(|x| format!("{}_{}", x, suffix));
+                self.rename_impl(name)
+            }
+            None => self,
+        }
     }
 }
 
@@ -632,6 +651,16 @@ macro_rules! define_circuit_union_impl {
                     pub fn [<as_mut_ $x:snake>](&mut self) -> Option<&mut $crate::circuit::$x> {
                         self.as_mut_op()
                     }
+
+                    pub fn [<into_ $x:snake _unwrap>](self) -> $crate::circuit::$x {
+                        self.into_unwrap()
+                    }
+                    pub fn [<as_ $x:snake _unwrap>](&self) -> &$crate::circuit::$x {
+                        self.as_unwrap()
+                    }
+                    pub fn [<as_mut_ $x:snake _unwrap>](&mut self) -> &mut $crate::circuit::$x {
+                        self.as_mut_unwrap()
+                    }
                 }
                 // Easy to also add macro to implement AsOp for pairs of enums to downcast.
                 impl AsOp<$crate::circuit::$x> for $name {
@@ -777,6 +806,24 @@ macro_rules! define_circuit_union_impl {
 macro_rules! define_circuit {
     [$($x:ident),+ $(,)?] => {
         define_circuit_union_impl!(Circuit {$($x,)*});
+
+        // you have to wrap pymethods in the paste for this to work due to how
+        // pymethods proc macro works
+        paste::paste! {
+            #[pymethods]
+            impl PyCircuitBase {
+            $(
+                pub fn [<as_ $x:snake>](&self) -> Option<$crate::circuit::$x> {
+                    arc_ref_clone(&self).into_op()
+                }
+
+                pub fn [<as_ $x:snake _unwrap>](&self) -> $crate::circuit::$x {
+                    arc_ref_clone(&self).into_unwrap()
+                }
+            )*
+            }
+        }
+
     }
 }
 
@@ -838,7 +885,7 @@ macro_rules! define_circuit_union {
         paste::paste! {
             #[pyfunction]
             pub fn [<circuit_is_ $name:snake>](circuit:$crate::circuit::CircuitRc)->bool{
-                let op: ::std::option::Option<$name>=(**circuit).clone().into();
+                let op: ::std::option::Option<$name>=$crate::util::arc_unwrap_or_clone(circuit.0).into();
                 op.is_some()
             }
         }
@@ -888,11 +935,7 @@ macro_rules! circuit_node_extra_impl {
         impl $type_name {
             fn into_init(self) -> PyClassInitializer<Self> {
                 // kinda awkward clone... (but probably basically free)
-                (
-                    self.clone(),
-                    $crate::circuit::PyCircuitBase(::std::sync::Arc::new(self.c())),
-                )
-                    .into()
+                (self.clone(), $crate::circuit::PyCircuitBase(self.rc())).into()
             }
         }
 
@@ -1604,7 +1647,7 @@ where
     }
     let inner_mapped = circuit.map_children_op(|x| deep_map_op_context(x, f, context, self_cache));
     let result = match inner_mapped {
-        Some(z) => f(z.clone().rc(), context).or(Some(z.rc())),
+        Some(z) => f(z.crc(), context).or(Some(z.rc())),
         None => f(circuit.clone(), context),
     };
     self_cache.insert(circuit.info().hash, result.clone());
@@ -1741,7 +1784,7 @@ impl CircuitNodeDefer for CircuitRc {
     }
 
     fn custom_c(self) -> Circuit {
-        (**self).clone()
+        arc_unwrap_or_clone(self.0)
     }
 
     // fast custom impl
@@ -1752,10 +1795,10 @@ impl CircuitNodeDefer for CircuitRc {
 
 #[pyclass(subclass, name = "Circuit")]
 #[derive(Clone, Debug)]
-pub struct PyCircuitBase(Arc<Circuit>);
+pub struct PyCircuitBase(CircuitRc);
 
 impl Deref for PyCircuitBase {
-    type Target = Arc<Circuit>;
+    type Target = CircuitRc;
 
     fn deref(&self) -> &Self::Target {
         &self.0
@@ -1768,17 +1811,6 @@ impl DerefMut for PyCircuitBase {
     }
 }
 
-fn use_rust_comp<T: PartialOrd>(l: &T, r: &T, comp_op: CompareOp) -> bool {
-    match comp_op {
-        CompareOp::Lt => l < r,
-        CompareOp::Gt => l > r,
-        CompareOp::Le => l <= r,
-        CompareOp::Ge => l >= r,
-        CompareOp::Eq => l == r,
-        CompareOp::Ne => l != r,
-    }
-}
-
 #[derive(Clone, Debug, FromPyObject)]
 enum PyReductionAxes {
     Single(i64),
@@ -1857,7 +1889,7 @@ impl PyCircuitBase {
     }
 
     pub fn total_flops(&self) -> BigUint {
-        total_flops((*self.0).clone().rc())
+        total_flops((*self.0).crc())
     }
 
     pub fn max_non_input_size(&self) -> BigUint {
@@ -1892,25 +1924,26 @@ impl PyCircuitBase {
     }
 
     fn to_py(&self) -> PyObject {
-        circuit_rust_to_py(CircuitRc(self.0.clone()))
+        circuit_rust_to_py(self.0.clone())
     }
 
     #[args(device_dtype = "Default::default()")]
     fn evaluate(&self, device_dtype: TorchDeviceDtypeOp) -> Result<Tensor, TensorEvalError> {
-        evaluate_fn_dtype_device((***self).clone().rc(), device_dtype)
+        evaluate_fn_dtype_device(self.0.clone(), device_dtype)
     }
 
-    fn map_children_enumerate(&self, f: PyObject) -> Result<Circuit, CircuitConstructionError> {
-        (&**self)
+    fn map_children_enumerate(&self, f: PyObject) -> Result<CircuitRc, CircuitConstructionError> {
+        self.0
             .map_children_enumerate(|i, child| pycall!(f, (i, child), CircuitConstructionError))
     }
 
-    fn map_children(&self, f: PyObject) -> Result<Circuit, CircuitConstructionError> {
-        (&**self).map_children(|child| pycall!(f, (child,), CircuitConstructionError))
+    fn map_children(&self, f: PyObject) -> Result<CircuitRc, CircuitConstructionError> {
+        self.0
+            .map_children(|child| pycall!(f, (child,), CircuitConstructionError))
     }
 
     fn total_arrayconstant_size(&self) -> BigUint {
-        total_arrayconstant_size(CircuitRc((**self).clone()))
+        total_arrayconstant_size(self.0.clone())
     }
 
     fn get_compatible_device_dtype(&self) -> TorchDeviceDtype {
@@ -1918,11 +1951,14 @@ impl PyCircuitBase {
     }
 
     fn rename(&self, name: Option<String>) -> CircuitRc {
-        (***self).clone().rename(name).rc()
+        self.0.clone().rename(name)
+    }
+    fn add_suffix(&self, suffix: Option<&str>) -> CircuitRc {
+        self.0.clone().add_suffix(suffix)
     }
 
     fn visit(&self, f: PyObject) -> Result<(), CircuitConstructionError> {
-        visit_circuit_py(CircuitRc((**self).clone()), f)
+        visit_circuit_py(self.0.clone(), f)
     }
 
     fn reduce(
