@@ -483,7 +483,7 @@ impl ModuleNodeSpec {
                     );
                     real_input_specs[i] = Some(argspec);
                     return (
-                        Some(real_input_specs[i].as_ref().unwrap().symbol.clone().rc()),
+                        Some(real_input_specs[i].as_ref().unwrap().symbol.crc()),
                         true,
                     );
                 }
@@ -520,7 +520,7 @@ impl ModuleNodeSpec {
             self.input_specs
                 .iter()
                 .position(|x| x.symbol.info().hash == c.info().hash)
-                .map(|p| input_specs[p].symbol.clone().rc())
+                .map(|p| input_specs[p].symbol.crc())
         })?;
         Ok(Self {
             spec_circuit,
