@@ -1,5 +1,5 @@
 use crate::all_imports::OpSize;
-use crate::circuit::{Add, Einsum, ScalarConstant};
+use crate::circuit::{Add, Einsum};
 use crate::util::filter_out_idx;
 use crate::{pyo3_prelude::*, sv};
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
