@@ -1,19 +1,12 @@
 use smallvec::Array;
 
-use crate::{
-    all_imports::Shape,
-    hashmaps::{AHashSet as HashSet, FxHashMap as HashMap},
-    smallvec::Sv,
-    util::cumsum,
-};
-use std::iter::zip;
-
 use super::{
     algebraic_rewrite::{get_removable_axes, remove_axes},
     prelude::*,
     Add, Concat, Einsum, Index, Rearrange, ScalarConstant, Scatter,
 };
 use crate::pyo3_prelude::*;
+use crate::{all_imports::Shape, util::cumsum};
 use crate::{
     rearrange_spec::RearrangeSpec,
     tensor_util::{
@@ -21,7 +14,10 @@ use crate::{
     },
     util::filter_out_idx,
 };
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use smallvec::SmallVec as Sv;
 use std::hash::Hash;
+use std::iter::zip;
 
 /// right now writing scatter_fuse, einsum_pull_scatter, add_pull_scatter
 #[pyfunction]
@@ -63,7 +59,7 @@ where
 pub fn einsum_pull_scatter(einsum: &Einsum) -> Option<CircuitRc> {
     let mut did_anything = false;
     let int_sizes = einsum.shape_map().unwrap();
-    let mut int_slices: HashMap<u8, USlice> = HashMap::new();
+    let mut int_slices: HashMap<u8, USlice> = HashMap::default();
     for (node, ints) in &einsum.args {
         match &***node {
             Circuit::Scatter(scatter) => {
@@ -204,7 +200,7 @@ pub fn scatter_elim_identity(scatter: &Scatter) -> Option<CircuitRc> {
 pub fn index_einsum_to_scatter(node: &Index) -> Option<CircuitRc> {
     let mut did_anything = false;
     if let Circuit::Einsum(inner) = &**node.node {
-        let mut int_slices: HashMap<u8, USlice> = HashMap::new();
+        let mut int_slices: HashMap<u8, USlice> = HashMap::default();
         let canon_index = node.index.canonicalize(&inner.info().shape);
         let containing_uslices: Vec<Option<USlice>> = canon_index
             .0
