@@ -1,3 +1,5 @@
+use std::fmt::{self, Display};
+
 use super::{
     circuit_optimizer::{OptimizationContext, OptimizationSettings},
     circuit_utils::{hash_to_node, toposort_circuit},
@@ -9,7 +11,6 @@ use super::{
     visit_circuit_unwrap, Add, ArrayConstant, HashBytes, IrreducibleNode, Leaf, ModuleNodeSpec,
     ScalarConstant, Symbol,
 };
-use crate::circuit::schedule_send::ScheduleToSend;
 use crate::lru_cache::TensorCacheRrfs;
 use crate::py_types::{scalar_to_tensor, tensor_scale, un_flat_concat, PY_UTILS};
 use crate::pyo3_prelude::*;
@@ -21,18 +22,14 @@ use crate::{
     tensor_util::{Shape, TorchDeviceDtype},
     timed,
 };
+use crate::{circuit::schedule_send::ScheduleToSend, util::arc_ref_clone};
 use itertools::Itertools;
 use macro_rules_attribute::apply;
 use miniserde::json;
 use num_bigint::BigUint;
-use pyo3::{
-    create_exception,
-    exceptions::PyException,
-    types::{PyByteArray, PyBytes},
-};
+use pyo3::{create_exception, exceptions::PyException};
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-use smallvec::SmallVec as Sv;
-use std::fmt::{self, Display};
+
 use thiserror::Error;
 
 #[derive(Clone, Debug)]
@@ -225,7 +222,7 @@ impl Schedule {
         tensor_cache: &mut Option<TensorCacheRrfs>,
     ) -> Result<Self, CircuitConstructionError> {
         let sent: ScheduleToSend = json::from_str(&string)
-            .map_err(|e| CircuitConstructionError::ParsingFail { string: string })?;
+            .map_err(|_e| CircuitConstructionError::ParsingFail { string })?;
         sent.load(device, tensor_cache)
     }
 }
@@ -920,7 +917,7 @@ pub fn circuit_to_schedule(
         &circuit_order,
         &to_node
             .iter()
-            .filter_map(|x| Option::<IrreducibleNode>::from((***x.1).clone()))
+            .filter_map(|x| Option::<IrreducibleNode>::from(arc_ref_clone(&x.1)))
             .collect(),
         &to_node
             .iter()
