@@ -63,10 +63,6 @@ macro_rules! sv {
         smallvec::smallvec!($($tt)*)
     };
 }
-#[pyfunction]
-fn demo_panicing_function(x: circuit::CircuitRc) {
-    panic!();
-}
 
 /// A Python module implemented in Rust. The name of this function must match
 /// the `lib.name` setting in the `Cargo.toml`, else Python will not be able to
@@ -160,8 +156,11 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_class::<circuit::get_update::IterativeMatcher>()?;
     m.add_class::<circuit::get_update::IterateMatchResults>()?;
     m.add_class::<circuit::get_update::Transform>()?;
-    m.add_class::<circuit::get_update::Updator>()?;
+    m.add_class::<circuit::get_update::Updater>()?;
+    m.add_class::<circuit::get_update::BoundUpdater>()?;
     m.add_class::<circuit::get_update::Getter>()?;
+    m.add_class::<circuit::get_update::BoundGetter>()?;
+    m.add_class::<circuit::get_update::Batcher>()?;
 
     m.add_class::<lru_cache::TensorCacheRrfs>()?;
 
@@ -356,6 +355,5 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     )?)?;
 
     m.add_function(wrap_pyfunction!(circuit::print_circuit_type_check, m)?)?;
-    m.add_function(wrap_pyfunction!(demo_panicing_function, m)?)?;
     Ok(())
 }
