@@ -576,7 +576,7 @@ impl EinsumSpec {
             subgraph_results.push(dp.last().unwrap()[&sg_bmp].clone());
         }
         let overall_contraction = independent_tree_contraction(
-            &subgraph_results.iter().map(|x| (**x).clone()).collect(),
+            &subgraph_results.iter().map(|x| (**x).clone()).collect(), /* maybe should use rc_unwrap_or_clone */
             &tensor_bmps_used,
             all_operands_bmp,
             out_bmp,
