@@ -1,11 +1,8 @@
 use crate::{all_imports::TorchDeviceDtype, pyo3_prelude::*};
 use macro_rules_attribute::apply;
+use pyo3::pyclass::CompareOp;
 use pyo3::types::IntoPyDict;
-use pyo3::{
-    exceptions,
-    types::{PyBytes, PyTuple},
-};
-use uuid::Uuid;
+use pyo3::types::PyTuple;
 
 use crate::{
     circuit::{EinsumAxes, HashBytes},
@@ -16,8 +13,6 @@ use crate::{
 pub struct PyUtils {
     pub torch: PyObject,
     get_tensor_shape: PyObject,
-    get_uuid_bytes: PyObject,
-    construct_uuid_bytes: PyObject,
     id: PyObject,
     pub cast_int: PyObject,
     scalar_to_tensor: PyObject,
@@ -51,8 +46,6 @@ pub static PY_UTILS: GILLazyPy<PyUtils> = GILLazyPy::new_py(|py| {
     PyUtils {
         torch: get("torch"),
         get_tensor_shape: get("get_tensor_shape"),
-        get_uuid_bytes: get("get_uuid_bytes"),
-        construct_uuid_bytes: get("construct_uuid_bytes"),
         id: get("get_id"),
         cast_int: get("cast_int"),
         scalar_to_tensor: get("scalar_to_tensor"),
@@ -340,6 +333,11 @@ impl Tensor {
         self.hash.as_ref()
     }
 
+    /// DANGEROUS
+    pub fn set_hash(&mut self, hash: Option<HashBytes>) {
+        self.hash = hash;
+    }
+
     pub fn hash_usize(&self) -> Option<usize> {
         self.hash.as_ref().map(|x| {
             let mut hash_prefix: [u8; 8] = Default::default();
@@ -385,3 +383,14 @@ impl IntoPy<PyObject> for PyEinsumAxes {
         PyTuple::new(py, self.0).into_py(py)
     }
 }
+
+pub fn use_rust_comp<T: PartialOrd>(l: &T, r: &T, comp_op: CompareOp) -> bool {
+    match comp_op {
+        CompareOp::Lt => l < r,
+        CompareOp::Gt => l > r,
+        CompareOp::Le => l <= r,
+        CompareOp::Ge => l >= r,
+        CompareOp::Eq => l == r,
+        CompareOp::Ne => l != r,
+    }
+}
