@@ -1,10 +1,5 @@
-use core::mem::size_of;
-
 use crate::lru_cache::TensorCacheRrfs;
-use crate::{
-    circuit::scheduled_execution::Schedule, py_types::PY_UTILS, pycall, pyo3_prelude::*,
-    rrfs::save_tensor_rrfs,
-};
+use crate::{circuit::scheduled_execution::Schedule, py_types::PY_UTILS, pycall, pyo3_prelude::*};
 use tiny_http::{Response, Server};
 
 #[pyfunction]
