@@ -1,11 +1,10 @@
 use crate::make_lazy;
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
-use crate::python_println;
 use macro_rules_attribute::apply;
 use once_cell::sync::Lazy;
 
-use std::{fmt::Debug, hash::Hash, mem};
+use std::{fmt::Debug, hash::Hash, mem, rc::Rc, sync::Arc};
 pub type AxisInt = u8;
 
 /// letters used in einsum and rearrange strings
@@ -164,10 +163,25 @@ macro_rules! timed_value {
 }
 
 pub trait AsOp<T> {
-    fn into_op(self) -> Option<T>;
+    fn into_op(self) -> Option<T>
+    where
+        Self: Sized;
     fn as_op(&self) -> Option<&T>;
     fn as_mut_op(&mut self) -> Option<&mut T>;
 
+    fn into_unwrap(self) -> T
+    where
+        Self: Sized,
+    {
+        self.into_op().unwrap()
+    }
+    fn as_unwrap(&self) -> &T {
+        self.as_op().unwrap()
+    }
+    fn as_mut_unwrap(&mut self) -> &mut T {
+        self.as_mut_op().unwrap()
+    }
+
     fn map_or_clone<'a, F, O, OF>(&'a self, f: F) -> O
     where
         Self: Into<O>,
@@ -430,3 +444,41 @@ pub fn as_sorted<Item: Ord>(x: impl IntoIterator<Item = Item>) -> Vec<Item> {
     out.sort();
     out
 }
+
+// https://github.com/rust-lang/rust/pull/91589
+#[inline]
+pub fn arc_ref_clone<T: Clone>(x: &Arc<T>) -> T {
+    arc_unwrap_or_clone(x.clone())
+}
+#[inline]
+pub fn arc_unwrap_or_clone<T: Clone>(x: Arc<T>) -> T {
+    Arc::try_unwrap(x).unwrap_or_else(|arc| (*arc).clone())
+}
+#[inline]
+pub fn rc_unwrap_or_clone<T: Clone>(x: Rc<T>) -> T {
+    Rc::try_unwrap(x).unwrap_or_else(|rc| (*rc).clone())
+}
+
+#[macro_export]
+macro_rules! simple_from {
+    (|$var:ident : $a:ty| -> $b:ty { $($fill:tt)* }) => {
+        impl From<$a> for $b {
+            fn from($var: $a) -> Self {
+                $($fill)*
+            }
+        }
+
+    };
+}
+
+#[macro_export]
+macro_rules! simple_default {
+    ($a:ty { $($fill:tt)* }) => {
+        impl Default for $a {
+            fn default() -> Self {
+                $($fill)*
+            }
+        }
+
+    };
+}
